@@ -26,8 +26,10 @@ Module containing docker-related applications and functions
 import collections
 import logging
 import os
+import pwd
 import threading
 import time
+import json
 from overrides import overrides
 
 from configobj import ConfigObj
@@ -40,8 +42,7 @@ from ..exceptions import InvalidDropException
 
 logger = logging.getLogger(__name__)
 
-# DLG_ROOT = '/dlg_root'
-DLG_ROOT = ""
+DLG_ROOT = utils.getDlgDir()
 
 DockerPath = collections.namedtuple("DockerPath", "path")
 
@@ -67,6 +68,51 @@ class ContainerIpWaiter(object):
         return self._uid, self._containerIp
 
 
+##
+# @brief Docker
+# @details
+# @par EAGLE_START
+# @param category Docker
+# @param tag template
+# @param[in] cparam/image Image//String/readwrite/False//False/
+#     \~English The name of the docker image to be used for this application
+# @param[in] cparam/tag Tag/1.0/String/readwrite/False//False/
+#     \~English The tag of the docker image to be used for this application
+# @param[in] cparam/digest Digest//String/readwrite/False//False/
+#     \~English The hexadecimal hash (long version) of the docker image to be used for this application
+# @param[in] cparam/command Command//String/readwrite/False//False/
+#     \~English The command line to run within the docker instance. The specified command will be executed in a bash shell. That means that images will need a bash shell.
+# @param[in] cparam/input_redirection Input Redirection//String/readwrite/False//False/
+#     \~English The command line argument that specifies the input into this application
+# @param[in] cparam/output_redirection Output Redirection//String/readwrite/False//False/
+#     \~English The command line argument that specifies the output from this application
+# @param[in] cparam/command_line_arguments Command Line Arguments//String/readwrite/False//False/
+#     \~English Additional command line arguments to be added to the command line to be executed
+# @param[in] cparam/paramValueSeparator Param value separator/ /String/readwrite/False//False/
+#     \~English Separator character(s) between parameters on the command line
+# @param[in] cparam/argumentPrefix Argument prefix/"--"/String/readwrite/False//False/
+#     \~English Prefix to each keyed argument on the command line
+# @param[in] cparam/execution_time Execution Time/5/Float/readonly/False//False/
+#     \~English Estimated execution time
+# @param[in] cparam/num_cpus No. of CPUs/1/Integer/readonly/False//False/
+#     \~English Number of cores used
+# @param[in] cparam/group_start Group start/False/Boolean/readwrite/False//False/
+#     \~English Is this node the start of a group?
+# @param[in] cparam/input_error_threshold "Input error rate (%)"/0/Integer/readwrite/False//False/
+#     \~English the allowed failure rate of the inputs (in percent), before this component goes to ERROR state and is not executed
+# @param[in] cparam/n_tries Number of tries/1/Integer/readwrite/False//False/
+#     \~English Specifies the number of times the 'run' method will be executed before finally giving up
+# @param[in] cparam/user User//String/readwrite/False//False/
+#     \~English Username of the user who will run the application within the docker image
+# @param[in] cparam/ensureUserAndSwitch Ensure User And Switch/False/Boolean/readwrite/False//False/
+#     \~English Make sure the user specified in the User parameter exists and then run the docker container as that user
+# @param[in] cparam/removeContainer Remove Container/True/Boolean/readwrite/False//False/
+#     \~English Instruct Docker engine to delete the container after execution is complete
+# @param[in] cparam/additionalBindings Additional Bindings//String/readwrite/False//False/
+#     \~English Directories which will be visible inside the container during run-time. Format is srcdir_on_host:trgtdir_on_container. Multiple entries can be separated by commas.
+# @param[in] cparam/portMappings Port Mappings//String/readwrite/False//False/
+#     \~English Port mappings on the host machine
+# @par EAGLE_END
 class DockerApp(BarrierAppDROP):
     """
     A BarrierAppDROP that represents a process running in a container
@@ -191,6 +237,12 @@ class DockerApp(BarrierAppDROP):
         BarrierAppDROP.initialize(self, **kwargs)
 
         self._image = self._getArg(kwargs, "image", None)
+        self._env = self._getArg(kwargs, "env", None)
+        self._cmdLineArgs = self._getArg(kwargs, "command_line_arguments", "")
+        self._applicationArgs = self._getArg(kwargs, "applicationArgs", {})
+        self._argumentPrefix = self._getArg(kwargs, "argumentPrefix", "--")
+        self._paramValueSeparator = self._getArg(kwargs, \
+            "paramValueSeparator", " ")
         if not self._image:
             raise InvalidDropException(
                 self, "No docker image specified, cannot create DockerApp"
@@ -204,25 +256,36 @@ class DockerApp(BarrierAppDROP):
             )
 
         self._command = self._getArg(kwargs, "command", None)
-        if not self._command:
+
+        self._noBash = False
+        if not self._command or self._command[:2].strip() == "%%":
             logger.warning(
-                "No command specified. Assume that a default command is executed in the container"
+                "Assume a default command is executed in the container"
             )
-            # raise InvalidDropException(
-            #     self, "No command specified, cannot create DockerApp")
+            self._command = self._command.strip()[2:].strip() if self._command else ""
+            self._noBash = True
+            # This makes sure that we can retain any command defined in the image, but still be
+            # able to add any arguments straight after. This requires to use the placeholder string
+            # "%%" at the start of the command, else it is interpreted as a normal command.
 
-        # The user used to run the process in the docker container
-        # By default docker containers run as root, but we don't want to run
-        # a process using a different user because otherwise anything that that
-        # process writes to the filesystem
-        self._user = self._getArg(kwargs, "user", None)
+        # construct the actual command line from all application parameters
+        argumentString = droputils.serialize_applicationArgs(self._applicationArgs, \
+            self._argumentPrefix, self._paramValueSeparator)
+        # complete command including all additional parameters and optional redirects
+        cmd = f"{self._command} {argumentString} {self._cmdLineArgs} "
+        cmd = cmd.strip()
+        self._command = cmd
 
-        # In some cases we want to make sure the command in the container runs
-        # as a certain user, so we wrap up the command line in a small script
-        # that will create the user if missing and switch to it
-        self._ensureUserAndSwitch = self._getArg(
-            kwargs, "ensureUserAndSwitch", self._user is None
-        )
+        # The user used to run the process in the docker container is now always the user
+        # who originally started the DALiuGE process as well. The information is passed through
+        # from the host to the engine container (if run as docker) and then further to any
+        # container running as a component.
+
+        pw = pwd.getpwuid(os.getuid())
+        self._user = pw.pw_name # use current user by default
+        self._userid = pw.pw_uid
+        self._groupid = pw.pw_gid
+        logger.debug(f"User for docker container: {self._user} {self._userid}:{self._groupid}")
 
         # By default containers are removed from the filesystem, but people
         # might want to preserve them.
@@ -235,13 +298,24 @@ class DockerApp(BarrierAppDROP):
         self._portMappings = self._getArg(kwargs, "portMappings", "")
         logger.info(f"portMappings: {self._portMappings}")
 
+        self._shmSize = self._getArg(kwargs, "shmSize", "")
+        logger.info(f"shmSize: {self._shmSize}")
+
         # Additional volume bindings can be specified for existing files/dirs
         # on the host system. They are given either as a list or as a
         # comma-separated string
         self._additionalBindings = {}
-        bindings = self._getArg(kwargs, "additionalBindings", [])
-        bindings = bindings.split(",") if isinstance(bindings, str) else bindings
+        bindings = [f"{utils.getDlgDir()}:{utils.getDlgDir()}",
+                    f"{utils.getDlgDir()}/workspace/settings/passwd:/etc/passwd",
+                    f"{utils.getDlgDir()}/workspace/settings/group:/etc/group"
+        ]
+        additionalBindings = self._getArg(kwargs, "additionalBindings", [])
+        additionalBindings = additionalBindings.split(",") if isinstance(additionalBindings, str) \
+            else additionalBindings
+        bindings += additionalBindings
         for binding in bindings:
+            if len(binding) == 0:
+                continue
             if binding.find(":") == -1:
                 host_path = container_path = binding
             else:
@@ -273,12 +347,16 @@ class DockerApp(BarrierAppDROP):
             logger.debug("Image '%s' found, no need to pull it", self._image)
 
         # Check if the image specifies a working directory
-        # If it doesn't use the one provided by the user
+        # If it doesn't use the one provided by the user.
+        # If none is provided use the session directory
         inspection = c.api.inspect_image(self._image)
         logger.debug("Docker Image inspection: %r", inspection)
         self.workdir = inspection.get("ContainerConfig", {}).get("WorkingDir", None)
+        # self.workdir = None
+        self._sessionId = (self._dlg_session.sessionId if self._dlg_session else "")
         if not self.workdir:
-            self.workdir = self._getArg(kwargs, "workingDir", "/")
+            default_workingdir = os.path.join(utils.getDlgWorkDir(), self._sessionId)
+            self.workdir = self._getArg(kwargs, "workingDir", default_workingdir)
 
         c.api.close()
 
@@ -320,10 +398,12 @@ class DockerApp(BarrierAppDROP):
         fsInputs = {uid: i for uid, i in iitems if droputils.has_path(i)}
         fsOutputs = {uid: o for uid, o in oitems if droputils.has_path(o)}
         dockerInputs = {
-            uid: DockerPath(DLG_ROOT + i.path) for uid, i in fsInputs.items()
+#            uid: DockerPath(utils.getDlgDir() + i.path) for uid, i in fsInputs.items()
+            uid: DockerPath(i.path) for uid, i in fsInputs.items()
         }
         dockerOutputs = {
-            uid: DockerPath(DLG_ROOT + o.path) for uid, o in fsOutputs.items()
+#            uid: DockerPath(utils.getDlgDir() + o.path) for uid, o in fsOutputs.items()
+            uid: DockerPath(o.path) for uid, o in fsOutputs.items()
         }
         dataURLInputs = {uid: i for uid, i in iitems if not droputils.has_path(i)}
         dataURLOutputs = {uid: o for uid, o in oitems if not droputils.has_path(o)}
@@ -338,7 +418,7 @@ class DockerApp(BarrierAppDROP):
         else:
             cmd = ""
 
-        # We bind the inputs and outputs inside the docker under the DLG_ROOT
+        # We bind the inputs and outputs inside the docker under the utils.getDlgDir()
         # directory, maintaining the rest of their original paths.
         # Outputs are bound only up to their dirname (see class doc for details)
         # Volume bindings are setup for FileDROPs and DirectoryContainers only
@@ -347,16 +427,20 @@ class DockerApp(BarrierAppDROP):
             os.path.dirname(o.path) + ":" + os.path.dirname(dockerOutputs[uid].path)
             for uid, o in fsOutputs.items()
         ]
+        logger.debug("Input/output bindings: %r", binds)
         if (
             len(self._additionalBindings.items()) > 0
         ):  # else we end up with a ':' in the mounts list
+            logger.debug("Additional bindings: %r", self._additionalBindings)
             binds += [
                 host_path + ":" + container_path
                 for host_path, container_path in self._additionalBindings.items()
             ]
         binds = list(set(binds))  # make this a unique list else docker complains
-        if binds == [":"]:
-            binds = []
+        try:
+            binds.remove(':')
+        except:
+            pass
         logger.debug("Volume bindings: %r", binds)
 
         portMappings = {}
@@ -382,52 +466,69 @@ class DockerApp(BarrierAppDROP):
             cmd = cmd.replace("%containerIp[{0}]%".format(uid), ip)
             logger.debug("Command after IP replacement is: %s", cmd)
 
-        # If a user has been given, we run the container as that user. It is
-        # useful to make sure that the USER environment variable is set in those
-        # cases (e.g., casapy requires this to correctly operate)
-        user = self._user
+        # deal with environment variables
         env = {}
-        if user is not None:
-            env = {"USER": user}
+        env.update({
+            "DLG_UID": self._uid},
+        )
+        if self._dlg_session:
+            env.update({"DLG_SESSION_ID":self._dlg_session.sessionId})
+        if self._user is not None:
+            env.update({
+                "USER": self._user,
+                "DLG_ROOT": utils.getDlgDir()
+                })
+        if self._env is not None:
+            logger.debug(f"Found environment variable setting: {self._env}")
+            if self._env.lower() == "all": # pass on all environment variables from host
+                env.update(os.environ)
+            elif self._env[0] in ["{", "["]:
+                try:
+                    addEnv = json.loads(self._env)
+                except:
+                   logger.warning("Ignoring provided environment variables: Format wrong? Check documentation")
+                   addEnv = {}
+                if isinstance(addEnv, dict): # if it is a dict populate directly
+                    # but replace placeholders first
+                    for key in addEnv:
+                        value = droputils.replace_path_placeholders(
+                            addEnv[key], dockerInputs, dockerOutputs
+                        )
+                        value = droputils.replace_dataurl_placeholders(
+                            value, dataURLInputs, dataURLOutputs
+                        )
+                        addEnv[key] = value
+                    env.update(addEnv)
+                elif isinstance(addEnv, list): # if it is a list populate from host environment
+                    for e in addEnv:
+                        env.update(os.environ[e])
+            else:
+                logger.warning("Ignoring provided environment variables: Format wrong! Check documentation")
+        logger.debug(f"Adding environment variables: {env}")
 
-        if self._ensureUserAndSwitch is True:
-            # Append commands that will make sure a user is present with the
-            # same UID of the current user, and that the command that was
-            # supplied for this container runs as that user.
-            # Also make sure that the output will belong to that user
-            uid = os.getuid()
-            createUserAndGo = "id -u {0} &> /dev/null || adduser --uid {0} r; ".format(
-                uid
-            )
-            for dirname in set(
-                [os.path.dirname(x.path) for x in dockerOutputs.values()]
-            ):
-                createUserAndGo += 'chown -R {0}.{0} "{1}"; '.format(uid, dirname)
-            createUserAndGo += "su -l $(getent passwd {0} | cut -f1 -d:) -c /bin/bash -c 'cd {1}; {2}'".format(
-                uid, self.workdir, utils.escapeQuotes(cmd, doubleQuotes=False)
-            )
-
-            cmd = createUserAndGo
 
         # Wrap everything inside bash
-        if len(cmd) > 0:
+        if len(cmd) > 0 and not self._noBash:
             cmd = '/bin/bash -c "%s"' % (utils.escapeQuotes(cmd, singleQuotes=False))
             logger.debug("Command after user creation and wrapping is: %s", cmd)
         else:
-            logger.debug("No command specified, executing container without!")
+            logger.debug("executing container with default cmd and wrapped arguments")
+            cmd = f"{utils.escapeQuotes(cmd, singleQuotes=False)}"
 
         c = DockerApp._get_client()
 
+        logger.debug(f"Final user for container: {self._user}:{self._userid}")
         # Create container
         self.container = c.containers.create(
             self._image,
             cmd,
             volumes=binds,
             ports=portMappings,
-            user=user,
+            user=f"{self._userid}:{self._groupid}",
             environment=env,
             working_dir=self.workdir,
             init=True,
+            shm_size=self._shmSize,
             # auto_remove=self._removeContainer,
             detach=True,
         )
@@ -440,9 +541,6 @@ class DockerApp(BarrierAppDROP):
         self.container.start()
         logger.info("Started container %s", cId)
 
-        # Capture output
-        stdout = self.container.logs(stream=False, stdout=True, stderr=False)
-        stderr = self.container.logs(stream=False, stdout=False, stderr=True)
 
         # Figure out the container's IP and save it
         # Setting self.containerIp will trigger an event being sent to the
@@ -461,6 +559,13 @@ class DockerApp(BarrierAppDROP):
             self._exitCode = x
 
         end = time.time()
+
+        # Capture output
+        stdout = self.container.logs(stream=False, stdout=True, stderr=False)
+        stderr = self.container.logs(stream=False, stdout=False, stderr=True)
+        if isinstance(stdout, bytes):
+            stdout = stdout.decode()
+            stderr = stderr.decode()
         logger.info(
             "Container %s finished in %.2f [s] with exit code %d",
             cId,
