@@ -32,56 +32,74 @@ logger = logging.getLogger(__name__)
 
 
 class PlasmaFlightClient:
-    def __init__(self, socket: str, scheme: str = "grpc+tcp", connection_args={}):
+    """
+    Client for accessing plasma-backed arrow flight data server.
+    """
+    def __init__(self, socket: str, scheme: str = "grpc+tcp", connection_args: Optional[dict] = None):
         """
         Args:
             socket (str): The socket of the local plasma store
             scheme (str, optional): [description]. Defaults to "grpc+tcp".
             connection_args (dict, optional): [description]. Defaults to {}.
         """
-        self.plasma_client = plasma.connect(socket)
+        self.plasma_client: plasma.PlasmaClient = plasma.connect(socket)
         self._scheme = scheme
-        self._connection_args = connection_args
+        self._connection_args = {} if connection_args is None else connection_args
 
     def list_flights(self, location: str):
+        """
+        Retrieves a list of flights
+        """
         flight_client = paf.FlightClient(
             f"{self._scheme}://{location}", **self._connection_args
         )
         return flight_client.list_flights()
 
     def get_flight(
-        self, object_id: plasma.ObjectID, location: Optional[str]
+        self, object_id: plasma.ObjectID, location: str
     ) -> paf.FlightStreamReader:
+        """
+        Retreives an flight object stream
+        """
         descriptor = paf.FlightDescriptor.for_path(
             object_id.binary().hex().encode("utf-8")
         )
-        if location is not None:
-            logger.debug(
-                f"connecting to {self._scheme}://{location} with descriptor {descriptor}"
-            )
-            flight_client = paf.FlightClient(
-                f"{self._scheme}://{location}", **self._connection_args
-            )
-            info = flight_client.get_flight_info(descriptor)
-            for endpoint in info.endpoints:
-                logger.debug(f"using endpoint locations {endpoint.locations}")
-                return flight_client.do_get(endpoint.ticket)
-        else:
-            raise Exception("Location required")
 
-    def create(self, object_id: plasma.ObjectID, size: int):
+        logger.debug(
+            f"connecting to {self._scheme}://{location} with descriptor {descriptor}"
+        )
+        flight_client = paf.FlightClient(
+            f"{self._scheme}://{location}", **self._connection_args
+        )
+        info = flight_client.get_flight_info(descriptor)
+        for endpoint in info.endpoints:
+            logger.debug("using endpoint locations %s", endpoint.locations)
+            return flight_client.do_get(endpoint.ticket)
+
+    def create(self, object_id: plasma.ObjectID, size: int) -> plasma.PlasmaBuffer:
+        """
+        Creates an empty plasma buffer
+        """
         return self.plasma_client.create(object_id, size)
 
     def seal(self, object_id: plasma.ObjectID):
+        """Seals the plasma buffer marking it as readonly"""
         self.plasma_client.seal(object_id)
 
-    def put(self, data: memoryview, object_id: plasma.ObjectID):
+    def put_raw_buffer(self, data: memoryview, object_id: plasma.ObjectID):
+        """Puts  """
         self.plasma_client.put_raw_buffer(data, object_id)
 
-    def get(
-        self, object_id: plasma.ObjectID, owner: Optional[str] = None
+    def get_buffer(
+        self,
+        object_id: plasma.ObjectID,
+        owner: Optional[str] = None
     ) -> memoryview:
-        logger.debug(f"PlasmaFlightClient Get {object_id}")
+        """
+        Gets the plasma object from the local store if it's available,
+        otherwise queries the plasmaflight owner for the object.
+        """
+        logger.debug("PlasmaFlightClient Get %s", object_id)
         if self.plasma_client.contains(object_id):
             # first check if the local store contains the object
             [buf] = self.plasma_client.get_buffers([object_id])
@@ -91,13 +109,16 @@ class PlasmaFlightClient:
             reader = self.get_flight(object_id, owner)
             table = reader.read_all()
             output = BytesIO(table.column(0)[0].as_py()).getbuffer()
-            # cache output
-            self.put(output, object_id)
+            # cache buffer in local plasma
+            self.put_raw_buffer(output, object_id)
             return output
         else:
             raise KeyError("ObjectID not found", object_id)
 
     def exists(self, object_id: plasma.ObjectID, owner: Optional[str] = None) -> bool:
+        """
+        Returns true if the remote plasmaflight server contains the plasma object.
+        """
         # check cache
         if self.plasma_client.contains(object_id):
             return True
@@ -113,6 +134,6 @@ class PlasmaFlightClient:
                     )
                 )
                 return True
-            except:
+            except paf.FlightError:
                 return False
         return False
