@@ -24,6 +24,7 @@ from dlg.drop import (
     BarrierAppDROP,
     NgasDROP,
     InMemoryDROP,
+    SharedMemoryDROP,
     NullDROP,
     RDBMSDrop,
     ContainerDROP,
@@ -44,13 +45,24 @@ from dlg.meta import (
 # single output via SSH's scp protocol.
 # @par EAGLE_START
 # @param category PythonApp
-# @param[in] param/appclass Application Class/dlg.apps.scp.ScpApp/String/readonly/
+# @param tag daliuge
+# @param[in] cparam/appclass Application Class/dlg.apps.scp.ScpApp/String/readonly/False//False/
 #     \~English Application class
-# @param[in] param/remoteUser Remote User//String/readwrite/
+# @param[in] cparam/execution_time Execution Time/5/Float/readonly/False//False/
+#     \~English Estimated execution time
+# @param[in] cparam/num_cpus No. of CPUs/1/Integer/readonly/False//False/
+#     \~English Number of cores used
+# @param[in] cparam/group_start Group start/False/Boolean/readwrite/False//False/
+#     \~English Is this node the start of a group?
+# @param[in] cparam/input_error_threshold "Input error rate (%)"/0/Integer/readwrite/False//False/
+#     \~English the allowed failure rate of the inputs (in percent), before this component goes to ERROR state and is not executed
+# @param[in] cparam/n_tries Number of tries/1/Integer/readwrite/False//False/
+#     \~English Specifies the number of times the 'run' method will be executed before finally giving up
+# @param[in] aparam/remoteUser Remote User//String/readwrite/False//False/
 #     \~English Remote user address
-# @param[in] param/pkeyPath Private Key Path//String/readwrite/
+# @param[in] aparam/pkeyPath Private Key Path//String/readwrite/False//False/
 #     \~English Private key path
-# @param[in] param/timeout Timeout//Float/readwrite/
+# @param[in] aparam/timeout Timeout//Float/readwrite/False//False/
 #     \~English Connection timeout in seconds
 # @param[in] port/file File/PathBasedDrop/
 #     \~English Input file path
@@ -78,12 +90,14 @@ class ScpApp(BarrierAppDROP):
         "input onto its single output via SSHs scp protocol.",
         [
             dlg_batch_input(
-                "binary/*", [NgasDROP, InMemoryDROP, NullDROP, RDBMSDrop, ContainerDROP]
+                "binary/*",
+                [NgasDROP, InMemoryDROP, SharedMemoryDROP, NullDROP, RDBMSDrop, ContainerDROP]
             )
         ],
         [
             dlg_batch_output(
-                "binary/*", [NgasDROP, InMemoryDROP, NullDROP, RDBMSDrop, ContainerDROP]
+                "binary/*",
+                [NgasDROP, InMemoryDROP, SharedMemoryDROP, NullDROP, RDBMSDrop, ContainerDROP]
             )
         ],
         [dlg_streaming_input("binary/*")],
