@@ -25,14 +25,17 @@ Utility methods and classes to be used when interacting with DROPs
 
 import collections
 import io
+import json
 import logging
+import pickle
 import re
 import threading
 import traceback
+from typing import IO, Any, AsyncIterable, BinaryIO, Dict, Iterable, overload
 import numpy as np
 
 from dlg.ddap_protocol import DROPStates
-from dlg.drop import AppDROP, AbstractDROP
+from dlg.drop import AppDROP, AbstractDROP, DataDROP, PathBasedDrop
 from dlg.io import IOForURL, OpenMode
 from dlg import common
 from dlg.common import DropType
@@ -129,7 +132,7 @@ def allDropContents(drop, bufsize=4096):
     return buf.getvalue()
 
 
-def copyDropContents(source, target, bufsize=4096):
+def copyDropContents(source: DataDROP, target: DataDROP, bufsize=4096):
     """
     Manually copies data from one DROP into another, in bufsize steps
     """
@@ -264,17 +267,51 @@ def listify(o):
     return [o]
 
 
+# def save_pickle(drop: DataDROP, data: Any):
+#     """Saves a python object in pkl format"""
+#     pickle.dump(data, drop)
+
+
+# def load_pickle(drop: DataDROP) -> Any:
+#     """Loads a pkl formatted data object stored in a DataDROP.
+#     Note: does not support streaming mode.
+#     """
+#     buf = io.BytesIO()
+#     desc = drop.open()
+#     while True:
+#         data = drop.read(desc)
+#         if not data:
+#             break
+#         buf.write(data)
+#     drop.close(desc)
+#     return pickle.loads(buf.getbuffer())
+
+
+# async def save_pickle_iter(drop: DataDROP, data: Iterable[Any]):
+#     for obj in data:
+#         yield drop.write(obj)
+
+
+# async def load_pickle_iter(drop: PathBasedDrop) -> AsyncIterable:
+#     with open(drop.path, 'rb') as p:
+#         while p.peek(1):
+#             yield pickle.load(p)
+
+
 def save_numpy(drop, ndarray: np.ndarray, allow_pickle=False):
     """
-    Saves a numpy ndarray to a drop
+    Saves a numpy ndarray to a drop in npy format
     """
     bio = io.BytesIO()
+    # np.save accepts a "file-like" object which basically just requires
+    # a .write() method. Try np.save(drop, array)
     np.save(bio, ndarray, allow_pickle=allow_pickle)
     drop.write(bio.getbuffer())
 
-def load_numpy(drop, allow_pickle=False) -> np.ndarray:
+
+def load_numpy(drop: DataDROP, allow_pickle=False) -> np.ndarray:
     """
-    Loads a numpy ndarray from a drop
+    Loads a numpy ndarray from a drop in npy format
     """
     dropio = drop.getIO()
     dropio.open(OpenMode.OPEN_READ)
@@ -282,6 +319,26 @@ def load_numpy(drop, allow_pickle=False) -> np.ndarray:
     dropio.close()
     return res
 
+
+# def save_jsonp(drop: PathBasedDrop, data: Dict[str, object]):
+#     with open(drop.path, 'r') as f:
+#         json.dump(data, f)
+
+
+# def save_json(drop: DataDROP, data: Dict[str, object]):
+#     # TODO: support BinaryIO or TextIO interface from DataIO?
+#     json_string = json.dumps(data)
+#     drop.write(json_string.encode('UTF-8'))
+
+
+# def load_json(drop: DataDROP) -> dict:
+#     dropio = drop.getIO()
+#     dropio.open(OpenMode.OPEN_READ)
+#     data = json.loads(dropio.buffer())
+#     dropio.close()
+#     return data
+
+
 class DROPFile(object):
     """
     A file-like object (currently only supporting the read() operation, more to
@@ -445,6 +502,41 @@ def replace_dataurl_placeholders(cmd, inputs, outputs):
 
     return cmd
 
+def serialize_applicationArgs(applicationArgs, prefix='--', separator=' '):
+    """
+    Unpacks the applicationArgs dictionary and returns a string
+    that can be used as command line parameters.
+    """
+    if not isinstance(applicationArgs, dict):
+        logger.info("applicationArgs are not passed as a dict. Ignored!")
+    # construct the actual command line from all application parameters
+    args = []
+    pargs = []
+    positional = False
+    precious = False
+    for (name, vdict) in applicationArgs.items():
+        if vdict in [None, False, ""]:
+            continue
+        elif isinstance(vdict,bool):
+            value = ''
+        elif isinstance(vdict, dict):
+            precious = vdict["precious"]
+            value = vdict["value"]
+            if value in [None, False, ""] and not precious:
+                continue
+            positional = vdict["positional"]
+        # short and long version of keywords
+        if positional:
+            pargs.append(str(value).strip())
+        else:
+            if prefix == "--" and len(name) == 1:
+                arg = [f'-{name} {value}']
+            else:
+                arg = [f'{prefix}{name}{separator}{value}'.strip()]
+            args += arg
+        
+    return f"{' '.join(args + pargs)}" # add positional arguments to end of args
+
 
 # Easing the transition from single- to multi-package
 get_leaves = common.get_leaves
