@@ -20,6 +20,8 @@
 #    MA 02111-1307  USA
 #
 from abc import abstractmethod, ABCMeta
+from http.client import HTTPConnection
+from multiprocessing.sharedctypes import Value
 from overrides import overrides
 import io
 import logging
@@ -42,10 +44,13 @@ logger = logging.getLogger(__name__)
 
 
 class OpenMode:
+    """
+    Open Mode for Data Drops
+    """
     OPEN_WRITE, OPEN_READ = range(2)
 
 
-class DataIO(object):
+class DataIO():
     """
     A class used to read/write data stored in a particular kind of storage in an
     abstract way. This base class simply declares a number of methods that
@@ -65,11 +70,12 @@ class DataIO(object):
     """
 
     __metaclass__ = ABCMeta
+    _mode: Optional[OpenMode]
 
     def __init__(self):
         self._mode = None
 
-    def open(self, mode, **kwargs):
+    def open(self, mode: OpenMode, **kwargs):
         """
         Opens the underlying storage where the data represented by this instance
         is stored. Depending on the value of `mode` subsequent calls to
@@ -78,7 +84,7 @@ class DataIO(object):
         self._mode = mode
         self._desc = self._open(**kwargs)
 
-    def write(self, data, **kwargs):
+    def write(self, data, **kwargs) -> int:
         """
         Writes `data` into the storage
         """
@@ -88,7 +94,7 @@ class DataIO(object):
             raise ValueError("Writing operation attempted on write-only DataIO object")
         return self._write(data, **kwargs)
 
-    def read(self, count, **kwargs):
+    def read(self, count: int, **kwargs):
         """
         Reads `count` bytes from the underlying storage.
         """
@@ -108,18 +114,21 @@ class DataIO(object):
         self._close()
         self._mode = None
 
-    def size(self, **kwargs):
+    def size(self, **kwargs) -> int:
         """
         Returns the current total size of the underlying stored object. If the
         storage class does not support this it is supposed to return -1.
         """
-        return _size(self)
+        return self._size(**kwargs)
 
     def isOpened(self):
+        """
+        Returns true if the io is currently opened for read or write.
+        """
         return self._mode is not None
 
     @abstractmethod
-    def exists(self):
+    def exists(self) -> bool:
         """
         Returns `True` if the data represented by this DataIO exists indeed in
         the underlying storage mechanism
@@ -131,7 +140,6 @@ class DataIO(object):
         Deletes the data represented by this DataIO
         """
 
-    @abstractmethod
     def buffer(self) -> Union[memoryview, bytes, bytearray, pyarrow.Buffer]:
         """
         Gets a buffer protocol compatible object of the drop data.
@@ -148,7 +156,7 @@ class DataIO(object):
         pass
 
     @abstractmethod
-    def _write(self, data, **kwargs):
+    def _write(self, data, **kwargs) -> int:
         pass
 
     @abstractmethod
@@ -156,7 +164,7 @@ class DataIO(object):
         pass
 
     @abstractmethod
-    def _size(self, **kwargs):
+    def _size(self, **kwargs) -> int:
         pass
 
 
@@ -171,21 +179,24 @@ class NullIO(DataIO):
     def _read(self, count=4096, **kwargs):
         return None
 
-    def _write(self, data, **kwargs):
+    def _write(self, data, **kwargs) -> int:
         return len(data)
 
     def _close(self, **kwargs):
         pass
 
-    def _size(self, **kwargs):
+    @overrides
+    def _size(self, **kwargs) -> int:
         """
         Size is always 0 for this storage class
         """
         return 0
 
-    def exists(self):
+    @overrides
+    def exists(self) -> bool:
         return True
 
+    @overrides
     def delete(self):
         pass
 
@@ -195,24 +206,31 @@ class ErrorIO(DataIO):
     An DataIO method that throws exceptions if any of its methods is invoked
     """
 
+    @overrides
     def _open(self, **kwargs):
         raise NotImplementedError()
 
+    @overrides
     def _read(self, count=4096, **kwargs):
         raise NotImplementedError()
 
-    def _write(self, data, **kwargs):
+    @overrides
+    def _write(self, data, **kwargs) -> int:
         raise NotImplementedError()
 
+    @overrides
     def _close(self, **kwargs):
         raise NotImplementedError()
 
-    def _size(self, **kwargs):
+    @overrides
+    def _size(self, **kwargs) -> int:
         raise NotImplementedError()
 
-    def exists(self):
+    @overrides
+    def exists(self) -> bool:
         raise NotImplementedError()
 
+    @overrides
     def delete(self):
         raise NotImplementedError()
 
@@ -231,39 +249,47 @@ class MemoryIO(DataIO):
     def _open(self, **kwargs):
         if self._mode == OpenMode.OPEN_WRITE:
             return self._buf
+        elif self._mode == OpenMode.OPEN_READ:
+            # TODO: potentially wasteful copy
+            return io.BytesIO(self._buf.getbuffer())
         else:
-            return io.BytesIO(self._buf.getvalue())
+            raise ValueError()
 
-    def _write(self, data, **kwargs):
+    @overrides
+    def _write(self, data, **kwargs) -> int:
         self._desc.write(data)
         return len(data)
 
+    @overrides
     def _read(self, count=4096, **kwargs):
         return self._desc.read(count)
 
+    @overrides
     def _close(self, **kwargs):
         if self._mode == OpenMode.OPEN_READ:
             self._desc.close()
         # If we're writing we don't close the descriptor because it's our
         # self._buf, which won't be readable afterwards
 
-    def _size(self, **kwargs):
+    @overrides
+    def _size(self, **kwargs) -> int:
         """
         Return actual size of user data rather than the whole Python object
         """
         return self._buf.getbuffer().nbytes
 
-    def exists(self):
+    @overrides
+    def exists(self) -> bool:
         return not self._buf.closed
 
+    @overrides
     def delete(self):
         self._buf.close()
 
     @overrides
     def buffer(self) -> memoryview:
-        #  TODO: This may also be an issue
-        return self._open().getbuffer()
-
+        # TODO: This may also be an issue
+        return self._buf.getbuffer()
 
 
 class SharedMemoryIO(DataIO):
@@ -278,6 +304,7 @@ class SharedMemoryIO(DataIO):
         self._pos = 0
         self._buf = None
 
+    @overrides
     def _open(self, **kwargs):
         self._pos = 0
         if self._buf is None:
@@ -286,7 +313,8 @@ class SharedMemoryIO(DataIO):
             self._buf = DlgSharedMemory(self._name)
         return self._buf
 
-    def _write(self, data, **kwargs):
+    @overrides
+    def _write(self, data, **kwargs) -> int:
         total_size = len(data) + self._written
         if total_size > self._buf.size:
             self._buf.resize(total_size)
@@ -296,12 +324,11 @@ class SharedMemoryIO(DataIO):
             self._buf.buf[self._written:total_size] = data
             self._written = total_size
             self._buf.resize(total_size)
-            """
-            It may be inefficient to resize many times, but assuming data is written 'once' this is
-            might be tolerable and guarantees that the size of the underlying buffer is tight.
-            """
+            # It may be inefficient to resize many times, but assuming data is written 'once' this is
+            # might be tolerable and guarantees that the size of the underlying buffer is tight.
         return len(data)
 
+    @overrides
     def _read(self, count=4096, **kwargs):
         if self._pos == self._buf.size:
             return None
@@ -312,20 +339,30 @@ class SharedMemoryIO(DataIO):
         self._pos = end
         return out
 
+    @overrides
     def _close(self, **kwargs):
         if self._mode == OpenMode.OPEN_WRITE:
             self._buf.resize(self._written)
         self._buf.close()
         self._buf = None
 
-    def exists(self):
+    @overrides
+    def _size(self, **kwargs) -> int:
+        return self._buf.size
+
+    @overrides
+    def exists(self) -> bool:
         return self._buf is not None
 
+    @overrides
     def delete(self):
         self._close()
 
 
 class FileIO(DataIO):
+    """
+    A file-based implementation of DataIO
+    """
     _desc: io.BufferedReader
     def __init__(self, filename, **kwargs):
         super(FileIO, self).__init__()
@@ -340,25 +377,34 @@ class FileIO(DataIO):
             # let the higher layer deal with this.
             pass
 
+    @overrides
     def _read(self, count=4096, **kwargs):
         return self._desc.read(count)
 
-    def _write(self, data, **kwargs):
+    @overrides
+    def _write(self, data, **kwargs) -> int:
         self._desc.write(data)
         return len(data)
 
+    @overrides
     def _close(self, **kwargs):
         self._desc.close()
 
-    def _size(self, **kwargs):
+    @overrides
+    def _size(self, **kwargs) -> int:
         return os.path.getsize(self._fnm)
 
     def getFileName(self):
+        """
+        Returns the drop filename
+        """
         return self._fnm
 
-    def exists(self):
+    @overrides
+    def exists(self) -> bool:
         return os.path.isfile(self._fnm)
 
+    @overrides
     def delete(self):
         os.unlink(self._fnm)
 
@@ -408,6 +454,7 @@ class NgasIO(DataIO):
             self._ngasSrv, self._ngasPort, self._ngasTimeout
         )
 
+    @overrides
     def _open(self, **kwargs):
         if self._mode == OpenMode.OPEN_WRITE:
             # The NGAS client API doesn't have a way to continually feed an ARCHIVE
@@ -418,6 +465,7 @@ class NgasIO(DataIO):
             self._writtenDataSize = 0
         return self._getClient()
 
+    @overrides
     def _close(self, **kwargs):
         client = self._desc
         if self._mode == OpenMode.OPEN_WRITE:
@@ -441,16 +489,19 @@ class NgasIO(DataIO):
         # Release the reference to _desc so the client object gets destroyed
         del self._desc
 
+    @overrides
     def _read(self, count, **kwargs):
         # Read data from NGAS and give it back to our reader
         self._desc.retrieve2File(self._fileId, cmd="QRETRIEVE")
 
-    def _write(self, data, **kwargs):
+    @overrides
+    def _write(self, data, **kwargs) -> int:
         self._buf += data
         self._writtenDataSize += len(data)
         return len(data)
 
-    def exists(self):
+    @overrides
+    def exists(self) -> bool:
         import ngamsLib  # @UnresolvedImport
 
         status = self._getClient().sendCmd("STATUS", pars=[["fileId", self._fileId]])
@@ -471,6 +522,10 @@ class NgasIO(DataIO):
         )
         return fs
 
+    @overrides
+    def _size(self, **kwargs) -> int:
+        return self._writtenDataSize
+
     def delete(self):
         pass  # We never delete stuff from NGAS
 
@@ -484,6 +539,7 @@ class NgasLiteIO(DataIO):
     The `ngaslite` module doesn't support the STATUS command yet, and because of
     that this class will throw an error if its `exists` method is invoked.
     """
+    _desc: HTTPConnection
 
     def __init__(
         self,
@@ -541,8 +597,7 @@ class NgasLiteIO(DataIO):
                 self._buf = None
             else:
                 logger.debug(
-                    "Length is known, assuming data has been sent (%s, %s)"
-                    % (self.fileId, self._length)
+                    f"Length is known, assuming data has been sent ({self._fileId}, {self._length})"
                 )
             ngaslite.finishArchive(conn, self._fileId)
             conn.close()
@@ -553,7 +608,7 @@ class NgasLiteIO(DataIO):
     def _read(self, count=4096, **kwargs):
         return self._desc.read(count)
 
-    def _write(self, data, **kwargs):
+    def _write(self, data, **kwargs) -> int:
         if self._is_length_unknown():
             self._buf += data
         else:
@@ -561,13 +616,14 @@ class NgasLiteIO(DataIO):
         logger.debug("Wrote %s bytes" % len(data))
         return len(data)
 
-    def exists(self):
+    def exists(self) -> bool:
         raise NotImplementedError("This method is not supported by this class")
 
     def fileStatus(self):
-        logger.debug("Getting status of file %s" % self._fileId)
+        logger.debug("Getting status of file %s", self._fileId)
         return ngaslite.fileStatus(self._ngasSrv, self._ngasPort, self._fileId)
 
+    @overrides
     def delete(self):
         pass  # We never delete stuff from NGAS
 
@@ -611,20 +667,50 @@ def IOForURL(url):
 
 
 class PlasmaIO(DataIO):
-    def __init__(self, object_id, plasma_path="/tmp/plasma"):
-        super(PlasmaIO, self).__init__()
+    """
+    A shared-memory IO reader/writer implemented using plasma store
+    memory buffers. Note: not compatible with PlasmaClient put()/get()
+    which performs data pickling before writing.
+    """
+    _desc: plasma.PlasmaClient
+
+    def __init__(
+        self,
+        object_id: plasma.ObjectID,
+        plasma_path="/tmp/plasma",
+        expected_size: Optional[int]=None,
+        use_staging=False
+    ):
+        """Initializer
+        Args:
+            object_id (plasma.ObjectID): 20 bytes unique object id
+            plasma_path (str, optional): The socket file path visible to all shared processes. Defaults to "/tmp/plasma".
+            expected_size (Optional[int], optional) Total size of data to allocate to buffer if known. Defaults to None.
+            use_staging (bool, optional): Whether to stream first to a resizable staging buffer. Defaults to False.
+        """
+        super().__init__()
         self._plasma_path = plasma_path
         self._object_id = object_id
         self._reader = None
         self._writer = None
+        # treat sizes <1 as None
+        self._expected_size = expected_size if expected_size and expected_size > 0 else None
+        self._buffer_size = 0
+        self._use_staging = use_staging
 
+    @overrides
     def _open(self, **kwargs):
         return plasma.connect(self._plasma_path)
 
+    @overrides
     def _close(self, **kwargs):
         if self._writer:
-            self._desc.put_raw_buffer(self._writer.getvalue(), self._object_id)
-            self._writer.close()
+            if self._use_staging:
+                self._desc.put_raw_buffer(self._writer.getbuffer(), self._object_id)
+                self._writer.close()
+            else:
+                self._desc.seal(self._object_id)
+                self._writer.close()
         if self._reader:
             self._reader.close()
 
@@ -634,18 +720,47 @@ class PlasmaIO(DataIO):
             self._reader = pyarrow.BufferReader(data)
         return self._reader.read1(count)
 
-    def _write(self, data, **kwargs):
-        if not self._writer:
-            # use client.create and FixedSizeBufferWriter
-            self._writer = pyarrow.BufferOutputStream()
+    @overrides
+    def _write(self, data, **kwargs) -> int:
+        """
+        Writes data into the PlasmaIO reserved buffer.
+        If use_staging is False and expected_size is None, only a single write is allowed.
+        If use_staging is False and expected_size is > 0, multiple writes up to expected_size is allowed.
+        If use_staging is True, any number of writes may occur with a small performance penalty.
+        """
+        # NOTE: data must be a collection of bytes for len to represent the buffer bytesize
+        assert isinstance(data, Union[memoryview, bytes, bytearray, pyarrow.Buffer].__args__)
+        databytes = data.nbytes if isinstance(data, memoryview) else len(data)
+
+        if self._use_staging:
+            if not self._writer:
+                # write into a resizable staging buffer
+                self._writer = io.BytesIO()
+        else:
+            if not self._writer:
+                # write directly into fixed size plasma buffer
+                self._buffer_size = self._expected_size if self._expected_size is not None else databytes
+                plasma_buffer = self._desc.create(self._object_id, self._buffer_size)
+                self._writer = pyarrow.FixedSizeBufferWriter(plasma_buffer)
+            if self._writer.tell() + databytes > self._buffer_size:
+                raise IOError("".join([f"attempted to write {self._writer.tell() + databytes} ",
+                                f"bytes to plasma buffer of size {self._buffer_size}, ",
+                                "consider using staging or expected_size argument"]))
+
         self._writer.write(data)
         return len(data)
 
-    def exists(self):
+    @overrides
+    def _size(self, **kwargs) -> int:
+        return self._buffer_size
+
+    @overrides
+    def exists(self) -> bool:
         return self._object_id in self._desc.list()
 
+    @overrides
     def delete(self):
-        pass
+        self._desc.delete([self._object_id])
 
     @overrides
     def buffer(self) -> memoryview:
@@ -654,64 +769,81 @@ class PlasmaIO(DataIO):
 
 
 class PlasmaFlightIO(DataIO):
+    """
+    A plasma drop managed by an arrow flight network protocol
+    """
+    _desc: PlasmaFlightClient
+
     def __init__(
         self,
         object_id: plasma.ObjectID,
         plasma_path="/tmp/plasma",
         flight_path: Optional[str] = None,
-        size: int = -1,
+        expected_size: Optional[int] = None,
+        use_staging = False
     ):
-        super(PlasmaFlightIO, self).__init__()
-        assert size >= -1
+        super().__init__()
         self._object_id = object_id
         self._plasma_path = plasma_path
         self._flight_path = flight_path
-        self._size = size
         self._reader = None
         self._writer = None
+        # treat sizes <1 as None
+        self._expected_size = expected_size if expected_size and expected_size > 0 else None
+        self._buffer_size = 0
+        self._use_staging = use_staging
 
     def _open(self, **kwargs):
-        self._done = False
         return PlasmaFlightClient(socket=self._plasma_path)
 
     def _close(self, **kwargs):
         if self._writer:
-            if self._size == -1:
-                self._desc.put(self._writer.getvalue(), self._object_id)
+            if self._use_staging:
+                self._desc.put_raw_buffer(self._writer.getbuffer(), self._object_id)
                 self._writer.close()
             else:
-                assert self._size == self._writer.tell()
+                if self._expected_size != self._writer.tell():
+                    logger.debug(f"written {self._writer.tell()} but expected {self._expected_size} bytes")
                 self._desc.seal(self._object_id)
         if self._reader:
             self._reader.close()
 
     def _read(self, count, **kwargs):
         if not self._reader:
-            data = self._desc.get(self._object_id, self._flight_path)
+            data = self._desc.get_buffer(self._object_id, self._flight_path)
             self._reader = pyarrow.BufferReader(data)
         return self._reader.read1(count)
 
-    def _write(self, data, **kwargs):
+    def _write(self, data, **kwargs) -> int:
+
+        # NOTE: data must be a collection of bytes for len to represent the buffer bytesize
+        assert isinstance(data, Union[memoryview, bytes, bytearray, pyarrow.Buffer].__args__)
+        databytes = data.nbytes if isinstance(data, memoryview) else len(data)
         if not self._writer:
-            if self._size == -1:
+            if self._use_staging:
                 # stream into resizeable buffer
-                logger.warning(
-                    "Using dynamically sized Plasma buffer. Performance may be reduced."
-                )
-                self._writer = pyarrow.BufferOutputStream()
+                logger.warning("Using dynamically sized Plasma buffer. Performance may be reduced.")
+                self._writer = io.BytesIO()
             else:
-                # optimally write directly to fixed size plasma buffer
-                self._buffer = self._desc.create(self._object_id, self._size)
-                self._writer = pyarrow.FixedSizeBufferWriter(self._buffer)
+                # write directly to fixed size plasma buffer
+                self._buffer_size = self._expected_size if self._expected_size is not None else databytes
+                plasma_buffer = self._desc.create(self._object_id, self._buffer_size)
+                self._writer = pyarrow.FixedSizeBufferWriter(plasma_buffer)
         self._writer.write(data)
         return len(data)
 
-    def exists(self):
+    @overrides
+    def exists(self) -> bool:
         return self._desc.exists(self._object_id, self._flight_path)
 
+    @overrides
+    def _size(self, **kwargs) -> int:
+        return self._buffer_size
+
+    @overrides
     def delete(self):
         pass
 
     @overrides
     def buffer(self) -> memoryview:
-        return self._desc.get(self._object_id, self._flight_path)
+        return self._desc.get_buffer(self._object_id, self._flight_path)
