@@ -152,10 +152,15 @@ class Session(object):
         if self._nm is not None:
             logdir = self._nm.logdir
         logfile = generateLogFileName(logdir, self.sessionId)
-        self.file_handler = logging.FileHandler(logfile)
-        self.file_handler.setFormatter(fmt)
-        self.file_handler.addFilter(SessionFilter(self.sessionId))
-        logging.root.addHandler(self.file_handler)
+        try:
+            self.file_handler = logging.FileHandler(logfile)
+            self.file_handler.setFormatter(fmt)
+            self.file_handler.addFilter(SessionFilter(self.sessionId))
+            logging.root.addHandler(self.file_handler)
+        except AttributeError as e:
+            print(e)
+        except FileNotFoundError as f:
+            print(f)
 
     @property
     def sessionId(self):
@@ -524,8 +529,11 @@ class Session(object):
         return dict(self._graph)
 
     def destroy(self):
-        self.file_handler.close()
-        logging.root.removeHandler(self.file_handler)
+        try:
+            self.file_handler.close()
+            logging.root.removeHandler(self.file_handler)
+        except AttributeError as e:
+            print(e)
 
     __del__ = destroy
 
