@@ -30,6 +30,23 @@ from .io import ErrorIO
 from .meta import dlg_string_param, dlg_list_param
 
 
+##
+# @brief S3
+# @details A 'bucket' object available on Amazon's Simple Storage Service (S3)
+# @par EAGLE_START
+# @param category S3
+# @param tag template
+# @param[in] cparam/data_volume Data volume/5/Float/readwrite/False//False/
+#     \~English Estimated size of the data contained in this node
+# @param[in] cparam/group_end Group end/False/Boolean/readwrite/False//False/
+#     \~English Is this node the end of a group?
+# @param[in] cparam/bucket Bucket//String/readwrite/False//False/
+#     \~English The S3 Bucket
+# @param[in] cparam/object_name Object Name//String/readwrite/False//False/
+#     \~English The S3 Object
+# @param[in] cparam/profile_name Profile Name//String/readwrite/False//False/
+#     \~English The S3 Profile
+# @par EAGLE_END
 class S3DROP(AbstractDROP):
     """
     A DROP that points to data stored in S3
@@ -56,7 +73,7 @@ class S3DROP(AbstractDROP):
         return "{}/{}".format(self.bucket, self.key)
 
     @property
-    def dataURL(self):
+    def dataURL(self) -> str:
         return "s3://{}/{}".format(self.bucket, self.key)
 
     def exists(self):
