@@ -27,6 +27,7 @@ import random
 import shutil
 import sqlite3
 import string
+import sys
 import tempfile
 import subprocess
 
@@ -36,6 +37,7 @@ from dlg.drop import (
     FileDROP,
     AppDROP,
     InMemoryDROP,
+    SharedMemoryDROP,
     PlasmaDROP,
     PlasmaFlightDROP,
     NullDROP,
@@ -91,6 +93,10 @@ class SumupContainerChecksum(BarrierAppDROP):
 
 
 class TestDROP(unittest.TestCase):
+    """
+    DataDROP related unit tests
+    """
+
     def setUp(self):
         """
         library-specific setup
@@ -131,6 +137,20 @@ class TestDROP(unittest.TestCase):
         """
         self._test_dynamic_write_withDropType(InMemoryDROP)
 
+    @unittest.skipIf(sys.version_info < (3, 8), "Shared memory does nt work < python 3.8")
+    def test_write_SharedMemoryDROP(self):
+        """
+        Test a SharedMemoryDROP with simple AppDROP (for checksum calculation)
+        """
+        self._test_write_withDropType(SharedMemoryDROP)
+
+    @unittest.skipIf(sys.version_info < (3, 8), "Shared memory does nt work < python 3.8")
+    def test_dynamic_write_SharedMemoryDROP(self):
+        """
+        Test a SharedMemoryDROP with simple AppDROP (for checksum calculation)
+        """
+        self._test_dynamic_write_withDropType(SharedMemoryDROP)
+
     def test_write_plasmaDROP(self):
         """
         Test an PlasmaDrop and a simple AppDROP (for checksum calculation)
@@ -167,7 +187,7 @@ class TestDROP(unittest.TestCase):
         finally:
             store.terminate()
 
-    def test__dynamic_write_plasmaFlightDROP(self):
+    def test_dynamic_write_plasmaFlightDROP(self):
         """
         Test an PlasmaDrop and a simple AppDROP (for checksum calculation)
         """
@@ -208,7 +228,8 @@ class TestDROP(unittest.TestCase):
         without an expected drop size (for app compatibility and not
         recommended in production)
         """
-        a = dropType("oid:A", "uid:A", expectedSize=-1)
+        # NOTE: use_staging required for multiple writes to plasma drops
+        a = dropType("oid:A", "uid:A", expectedSize=-1, use_staging=True)
         b = SumupContainerChecksum("oid:B", "uid:B")
         c = InMemoryDROP("oid:C", "uid:C")
         b.addInput(a)
